#include "box.h"
#include <cmath>
using namespace std;

// Constructors
Box::Box ()
{
    this->dim=0;
    this->data=NULL;
}
Box::Box (int dimension)
{
    this->dim=dimension;
    this->data=new Interval[dim];
    Interval empty(0);
    for (int i=0;i<=this->dim-1;i++) this->data[i]=empty;
}
Box::Box (Interval a, Interval b)
{   this->dim=2;
    this->data=new Interval[2];
    this->operator [](1)=a;
    this->operator [](2)=b;
}
Box::Box (Interval a, Interval b, Interval c)
{
    this->dim=3;
    this->data=new Interval[3];
    this->data[0]=a;
    this->data[1]=b;
    this->data[2]=c;
}
Box::Box (const Box &v)
{
    if (&v == this) return;
    this->dim=v.dim;
    this->data=new Interval[this->dim];
    for (int k=1; k<=this->dim; k++) this->operator [](k)=v[k];
}
// Destructor
Box::~Box () {delete [] data;}


// Members Overloaded Operators
Interval& Box::operator[] (int i)  const
{
    return this->data[i-1];
}
Box& Box::operator=(const Box &v)
                   {
    this->dim=v.dim;
    delete [] this->data;
    this->data=new Interval[this->dim];
    for (int k=1; k<=this->dim; k++) this->operator [](k)=v[k];
    return *this;
}
// Others Overloaded Operators
Box operator+(const Box& v,const Box& w)
{
    Box r(v.dim);
    for (int k=1; k<=r.dim; k++) r[k]=v[k]+w[k];
    return (r);
}
Box operator-(const Box& v)
{
    Box r(v.dim);
    for (int k=1; k<=r.dim; k++) r[k]=-v[k];
    return (r);
}
Box operator-(const Box& v,const Box& w)
{
    return (v+(-w));
}
Box operator*(const Interval& a,const Box& v)
{
    Box r(v.dim);
    for (int k=1; k<=r.dim; k++) r[k]=a*v[k];
    return (r);
}
Box	operator*(const Box& v,const Interval& a)
{
    return (a*v);
}
ostream& operator<<(ostream& os, const Box& v)
{
    cout << "Box \n";
    cout << "   dim="<<v.dim << endl;
    if (v.isEmpty()) os<<"Not a Box !"<< endl;
    for (int i=1; i<=v.dim; i++)
        os << "  " << i << ": "<< v[i] << endl;
    return (os);
}


// Other Members
int Box::getDim()
{
    return this->dim;
}
bool Box::isEmpty() const
{
    if (this->dim==0 || this->data==NULL) return true;
    for (int k=0; k<this->dim; k++)
    {
        if (this->data[k].isEmpty()) return true;
    }
    return false;
}
int Box::mainAxis() const
{
    double widthmax=this->operator [](1).width();
    int kmax=1;
    for (int k=2; k<=this->dim; k++)
    {
        if (this->operator [](k).width()>widthmax)
        {
            kmax=k;
            widthmax=this->operator [](k).width();
        };
    }
    return kmax;
}
double   Box::volume() const
{
    if (this->isEmpty()) return 0.0;
    double vol=1;
    for (int i=1;i<=this->dim;i++)
        vol=vol*this->operator [](i).width();
    return vol;
}
double Box::width() const
{
    if (this->isEmpty()) return 0.0;
    return this->operator [](this->mainAxis()).width();
}
// Friends Functions
void Bisect(Box& x, Box& x1, Box& x2)
{
    BisectAlong(x,x1,x2,x.mainAxis());
}
void BisectAlong(Box& x, Box& x1, Box& x2, int i)
{
    x1 = x;
    x2 = x;
    double mx = x[i].center();
    x1[i].setSup(mx);
    x2[i].setInf(mx);
}



