#include "window.h"
#include <QtGui>

#include "iboolean.h"
#include "interval.h"
#include "box.h"
#include "syntax_tree.h"
#include "sivia.h"
#include <fstream>
fstream info;

vector <Node> syntax_tree;

double precision=0.1;
bool stepbystep=false;
Frame *pavage1;


Window::Window()

{   info.open("info.txt",fstream::out);
    pavage1 = new Frame;
    pavage1Label = new QLabel("Pavage 1");
    createControls("Controls");
    createFunctionEdit("f(x1,x2)");
    QGridLayout *mainLayout = new QGridLayout;
    mainLayout->addWidget(functionGroup, 3, 0, Qt::AlignLeft);
    mainLayout->addWidget(pavage1, 0, 0, Qt::AlignLeft);
    mainLayout->addWidget(pavage1Label, 1, 0, Qt::AlignLeft);
    mainLayout->addWidget(controlsGroup, 2, 0, Qt::AlignLeft);
    setLayout(mainLayout);
    setWindowTitle("SIVIA solving f(x1,x2) < 0");
    createConnections();
}

Window::~Window()
{     info.close();
}

void Window::createControls(const QString &title)
{   controlsGroup = new QGroupBox(title);
    stepbystepCheck = new QCheckBox("Step By Step");
    stepbystepCheck->setChecked(false);
    precisionLabel = new QLabel("Precision :");
    precisionSpinBox = new QDoubleSpinBox;
    precisionSpinBox->setDecimals(3);
    precisionSpinBox->setSingleStep(0.02);
    precisionSpinBox->setRange(0.01,100);
    precisionSpinBox->setValue(0.1);
    bgo = new QPushButton("Go");
    bexit = new QPushButton("Exit");
    QHBoxLayout *controlsLayout = new QHBoxLayout;
    controlsLayout->addWidget(precisionLabel);
    controlsLayout->addWidget(precisionSpinBox);
    controlsLayout->addWidget(stepbystepCheck);
    controlsLayout->addWidget(bgo);
    controlsLayout->addWidget(bexit);
    controlsGroup->setLayout(controlsLayout);
}

void Window::createFunctionEdit(const QString &title)
{
    functionGroup = new QGroupBox(title);
    functionEdit=new QLineEdit("2*cos(x1)*sin(x2+x1)+1");
    functionLayout = new QGridLayout;
    functionLayout->addWidget(functionEdit,0,0);
    functionGroup->setLayout(functionLayout);
}


void Window::createConnections()
{   connect(bgo, SIGNAL(clicked()),this, SLOT(compute()));
    connect(bexit, SIGNAL(clicked()),this, SLOT(close()));
}

void Window::compute()
{   info<<"Start computing"<<'\n';
    precision=precisionSpinBox->value();
    stepbystep=stepbystepCheck->isChecked();
    Box X(Interval(-10,10),Interval(-10,10));
    //X[1]=Interval(-10,10);
    //X[2]=Interval(-10,10);
    int j = create_syntax_tree(functionEdit->text());
    pavage1->setPriorDomain(X);
    siviaFile(X);
}
